 /*+-------------------------------------------------------+
   |          SUGI 29 Sample                               |
   |          SAS MAPPING: Tips and tricks                 |
   |                                                       |
   |    NAME: Animate.sas                                  |
   |   TITLE: Example of creating an animated gif file.    |
   |          This example starts with zipcode.sas and adds|
   |          an animated driving path across the US thru  |
   |          some training centers.  It also introduces a |
   |          new way of doing zipcode lookup.             |
   | PRODUCT: GRAPH                                        |
   |                                                       |
   |   NOTES: Last Update 19FEB04                          |
   |          This example does NOT use ODS                |
   +-------------------------------------------------------+*/

%let dev=gifanim;
%let name=animate;
filename odsout '.';
goptions reset=global ;
/** Sets goptions xpixels and ypixels **/
%inc '../setsize.sas';

/***********************************************************************/
/* Data for the start and stop zipcodes of your trip each day along    */
/* with your daily expense, miles covered and hours driven.            */    
/***********************************************************************/    
data trip;
  length fromzip 7. tozip 7. expense miles time $ 6.;
  retain exptot 0 milestot 0;
  input fromzip tozip expense miles time;
  exptot=exptot+expense; /* total up expenses */
  milestot=milestot+miles; /* total up miles */
  cards;
27513 30326 206 400 6:34
30326 63102 243 533 8:46
63102 66210 173 261 3:58
66210 80111 282 604 9:06
80111 85012 251 896 13:27
85012 92614 315 361 5:36
;

/***********************************************************************/  
/* Match the to/from zipcode to x,y locations on the map and create    */
/* an annotate dataset for each day's travel.                          */  
/* Zipcode matching by using the KEY= on the Set statement             */
/***********************************************************************/    
data tripzip; 
  length function style color $ 8 position $ 1 text $ 20 ;
  retain xsys ysys '2' hsys '3' when 'a' text '' color 'blue' size .6 
         style 'solid' position '5' day 0;
  anno_flag=2; /*so we can separate the datasets later*/ 
  set trip; 
  day=day+1; /*keep a day counter*/
  size = .1 * (exptot/100); /*size the line by expenses of the trip*/

  /*get the from zipcode x and y values*/
  zip = fromzip; 
  link look_it_up; 
  function='MOVE';
  output;

  /*get the to zipcode x and y values*/     
  zip = tozip; 
  link look_it_up;  
  function='DRAW';
  output;     
  return; 

/* do a dataset lookup according to the provided zipcode with Key=*/
look_it_up:;  
     set sashelp.zipcode key=zip/unique; 
     /*correct x,y values for this map*/
     x=atan(1)/45 * x *-1;
     y=atan(1)/45 * y;
     return;
 run;


/***********************************************************************/    
/* Get your SAS training center location data with zipcode and phone.  */
/* Substitute your own zipcode based location data here.               */
/***********************************************************************/    
data trainzip;
  length zip 5. locationcode $ 2. phone $ 12.;
  /* zipcode of sas training centers, location code and phone */
  input zip  locationcode $ phone $;
  cards;
30326 AT 404-814-2560
78726 AU 512-258-5171
07921 BE 908-470-0080
02199 BO 617-262-4201
97006 BV 503-617-7100
27513 CA 919-677-8000
60601 CH 312-819-6800
45242 CI 513-563-9444
44131 CL 216-642-0641
75207 DA 214-977-3921
48075 DE 248-354-3008
80111 EN 303-290-9112
06033 HA 860-633-4119
77056 HO 713-355-2707
46250 IN 317-578-1700
66210 KC 913-491-1166
92614 LA 949-852-8550
33131 MI 305-381-9980
55402 MN 612-349-9023
10019 NY 212-757-3826
32746 OR 407-804-1995
15222 PB 412-227-6446
85012 PH 602-265-1616
19087 PI 610-640-0940
20850 RO 301-838-7030
98101 SE 206-681-6084
94104 SF 415-421-2227
95131 SJ 408-573-0902
63102 SL 314-421-6364
;
run;


/***********************************************************************/    
/* Sort the data by zipcode and then
/* get the X,Y centroid location for the zipcode by merging with the   */
/* zipcode dataset.  This will give you more information than you want */
/***********************************************************************/    
proc sort data=trainzip; 
  by zip;
run;
data temp(keep= zip locationcode phone x y city);  
  merge trainzip sashelp.zipcode;  
  by zip;
run;


/***********************************************************************/    
/* Now, drop the records that were not in the original data or         */
/* records that don't have a zipcode match. Adjust the lat/long to     */
/* match the map.                                                      */
/***********************************************************************/    
data trainzipxy; 
  set temp; 
    if (locationcode ^='' and x NE . AND y NE .) then do;
	  /*adjust the Lat/long values to match the map*/
      x=atan(1)/45 * x *-1;
      y=atan(1)/45 * y;
      output;
    end;
	/* put out a message for bad zipcodes */
	else if (locationcode ^='' and x=.) then do; /* couldn't find zipcode*/
	   put "WARNING: Check zipcode " zip " because it wasn't located.";
	   end;
run;


/************************************************************************/    
/* Create the annotate dataset with a red 'dot' at each zipcode location*/
/************************************************************************/    
data trainloc;
  set trainzipxy;
  length function style color $ 8 position $ 1 text $ 20 ;
  retain xsys ysys '2' hsys '3' when 'a' text '';
  anno_flag=1; /*so we can separate the datasets later*/
  /* Create a tool-tip for the dot */
  color='red'; size=.6; rotate=360; style='solid'; function='pie';
  position='5'; 
  output;
run;


/***********************************************************************/    
/* Reduce the size of the actual map data.                             */
/***********************************************************************/    
proc greduce data=maps.states out=work.states;
  id state;
run;
/************************************************************************/    
/* Keep only the 48 contigious states and actually write the reduced map*/   
/************************************************************************/    
proc sql;
  create table work.states as select * from work.states
    where fipstate(state) not in ('AK' 'HI' 'PR')
    and density < 4  /* this applies the greduce */
    ;
quit; run;


/***********************************************************************/    
/* Combine the map and annotate Data to project it the same.           */
/* Then split it apart again for display.                              */
/***********************************************************************/    
data work.combined; /* combine data*/
  set work.states trainloc tripzip; 
run;
/*project both datasets*/
proc gproject data=work.combined out=work.combined dupok;
  id state;
run;
/*split the data*/
data work.states trainloc days;
  set work.combined;
  if anno_flag=1 then output trainloc;
  else if anno_flag=2 then output days;
  else output work.states;
run;


/***********************************************************************/  
/* Split up the travel info by days and each day in a different        */
/* annotate dataset so we can see the movement of the trip.            */  
/***********************************************************************/    
data day1 day2 day3 day4 day5 day6;
  set days;
  /*Each dataset shows the trip up to that point.   */
  /*An easy modification here will cause it to only */
  /*show the single day travel                      */
  if day=6 then do;
       output day6;
	 end;
  else if day=5 then do;
       output day6;
	 output day5;
	 end;
  else if day=4 then do;
       output day6;
	 output day5;
	 output day4;
     end;
  else if day=3 then do;
       output day6;
	 output day5;
	 output day4;
	 output day3;
	 end;
  else if day=2 then do;
       output day6;
	 output day5;
	 output day4;
	 output day3;
	 output day2;
	 end;
  else do; 
       output day6;
	 output day5;
	 output day4;
	 output day3;
	 output day2;
	 output day1;
	 end;
  run;

/***********************************************************************/    
/* append the annotate dataset showing all training center locations   */
/***********************************************************************/   
%macro appanno(dataset); 
  data &dataset;
    set &dataset trainloc;
  run;
%mend appanno;

%appanno(day1);
%appanno(day2);
%appanno(day3);
%appanno(day4);
%appanno(day5);
%appanno(day6);



/*----------------------------------------------------------------*/
/* Macro to run GMAP with only the annotate dataset changing.     */
/*----------------------------------------------------------------*/
%macro do_map(anno);
  proc gmap data=work.states map=work.states anno=&anno;
     id state; 
     choro state / coutline=black  nolegend;
  run;
%mend do_map;
/*----------------------------------------------------------------*/


/***********************************************************************/  
/* Create a map for each day                                           */  
/***********************************************************************/    
   filename animap 'animap.gif';
   

   /*** Animate: Setup for Animation device ***/
   GOPTIONS DEVICE=&dev 
            gsfmode=replace gsfname=animap hsize=8 vsize=6 
            iteration=2 delay=150 disposal=background;


  pattern v=s c=CXE9E8DC r=100;
  goptions border;

  goptions htitle=2 htext=1;
  Title 'Trip Through Training Locations - Cary/Atlanta';
  %do_map(day1);

  /*** Animate: note that after the first image, you must 'append' ***/
  goptions gsfmode=append;
  Title 'Trip Through Training Locations - Atlanta/St Louis';
  %do_map(day2);

  Title 'Trip Through Training Locations - St Louis/Overland Park';
  %do_map(day3);

  Title 'Trip Through Training Locations - Overland Park/Denver';
  %do_map(day4);

  Title 'Trip Through Training Locations - Denver/Phoenix';
  %do_map(day5);

  /*** Animate: Note that the last image must have the epilog specified ***/
  goptions gepilog='3B'x ;
  Title 'Trip Through Training Locations - Phoenix/Irvine';
  %do_map(day6);

  quit;


/***********************************************************************/ 
/* Output a basic html file to run the gif image.                      */   
/***********************************************************************/    
data _null_;
   file "&name._&dev..html";
   put '<HTML>';
   put '<BODY>';
   put '<BLOCKQUOTE>';
   put '<P><img src="animap.gif" 
        alt="Trip Animation" ';
   put '</BLOCKQUOTE>';
   put '</BODY>';
   put '</HTML>';
run;


