 /*+-------------------------------------------------------+
   |          SUGI 29 Sample                               |
   |          SAS MAPPING: Tips and tricks                 |
   |                                                       |
   |    NAME: animarray.sas                                |
   |   TITLE: Start with microarray.sas and animate        |
   |          several results.                             |
   | PRODUCT: GRAPH                                        |
   |                                                       |
   |   NOTES: Last Update 24FEB04                          |
   |          This example does NOT use ODS                |
   +-------------------------------------------------------+*/

%let dev=gifanim;
%let name=animarray;
filename odsout '.';

/*cellsize of 1 has no padding between circles*/
%let cellsize=.75;  /*a number greater than zero and less than/equal 1*/
%let MAXCOLS=14;    /*grid size*/
%let MAXROWS=14;
%let CIRCLEINC=36;  /*10 points on the circle*/


goptions reset;

/** Sets goptions xpixels and ypixels **/
%inc '../setsize.sas';

/***********************************************************************/    
/* Create a map with round cells by the specified grid size.           */
/***********************************************************************/  
data mapdata (keep=x y location);
retain location 0;
/* 0,0 is the bottom left, so output in this order to */
/* make 1,1 the top left and 2,1 to be the second row, first col, etc*/
do b = &MAXROWS to 1 by -1;
   do a =1 to &MAXCOLS;
       location=location+1;  /*assign location in sequential order*/
       radius = &cellsize/2; 
       xcenter=a+radius;
       ycenter=b+radius;
       do degrees=0 to 360 by &CIRCLEINC;  
         radians=degrees/57.3; 
         x=(radius * cos(radians)) + xcenter;
         y=(radius * sin(radians)) + ycenter;
         output;
       end;
   end;
end;


/***********************************************************************/    
/* Read in response data to display on the grid and process it         */
/***********************************************************************/  
data sasuser.arraydata; /* What was tested */
    infile 'array.txt';
    retain location 0;
    length description $100;  length genbank $10.;
    input metarow 1-2 metacol 3-4 arrayrow 5-7 arraycol 8-10 symbol $ 12-19 description $ 20-116 genbank $  ;
run;

/*------------------------------------------------------------------*/
/* Macro to create dummy test data.                                 */
/*------------------------------------------------------------------*/
%macro CreateTest(dataset,seed1,seed2);
/*drop all but the first metaarray and add fake test results*/
data &dataset (drop=metarow metacol); 
   set sasuser.arraydata;
   if (metarow = 1 AND metacol = 1) then do;
       location= ((arrayrow-1)* &MAXCOLS ) + arraycol;
       /* Generate fake random data */
       outcome1 =(ranuni(&seed1) * 4 );  
       outcome2 =(ranuni(&seed1) * 4 ); 
       outcome3 =(ranuni(&seed2) * 4 ); 
       outcome4 =(ranuni(&seed2) * 4 ); 
       mean = (outcome1+outcome2+outcome3+outcome4)/4;
       wtmean= (outcome1* .5)+(outcome2* .2) + (outcome3* .2) + (outcome4 * .1);
       duration =int(ranuni(2) * 20 +1);
       value= int((ranuni(2) * 4 )+1);

       output;
      end;
   run;
%mend CreateTest;
/*------------------------------------------------------------------*/

/***********************************************************************/ 
/* Create 3 dummy tests for the animation.                             */   
/***********************************************************************/    
%CreateTest(response1,2,2);
%CreateTest(response2,5,5);
%CreateTest(response3,100,2);



/***********************************************************************/    
/* Setup 4 colors and legend                                           */
/***********************************************************************/  
/*set our red, yellow, green, black colors*/
pattern1  v=s c=cxff0000;
pattern2  v=s c=cxffff00;
pattern3  v=s c=cx00ff00;
pattern4  v=s c=cx000000;

legend1 
   position=(middle left) 
   across=1 
   offset=(3pct,0pct)
   frame cframe=gray 
   value=(color=white h=2.5pct

    tick=1 'sample DNA'
    tick=2 'combination'
    tick=3 'control DNA'
    tick=4 'neither'
   )
   /* shape of legend chicklets ... width, height */
   /* if you change the font, you'll have to adjust shape */
   shape=bar(3,1) 
   label=(color=white position=(top center) 'Color Key')
  ;



/* ---------------------------------------------------------*/     
/* Declare a macro to do the map, for the specified dataset */
/* ---------------------------------------------------------*/     
%macro do_map(dataset);
   proc gmap map=mapdata data=&dataset;
      id location;
      choro value / legend=legend1 ;
   run;
%mend do_map;
/* ---------------------------------------------------------*/     

/**********************************************************************/
/* Display the maps and create the animation.                         */
/**********************************************************************/
   filename anigif 'anigif.gif';
   /*Animate: Key options settings for animate */
   GOPTIONS DEVICE=&dev cback=black ctext=white
            gsfmode=replace gsfname=anigif hsize=8 vsize=6 
            iteration=2 delay=150 disposal=background;

   goptions ftitle=zapfb htitle=5pct ftext='arial' htext=3pct;
   Title1 "MicroArray Demo";
   %do_map(response1);

   /* Animate:  Must append other images */
   goptions gsfmode=append;
   %do_map(response2);

   /* Animate: Must put epilog on last image */
   goptions gepilog='3B'x;
   %do_map(response3);
 
/**********************************************************************/
/* Create a small, simple html for showing the animated gif.          */
/**********************************************************************/
data _null_;
   file "&name._&dev..html";
   put '<HTML>';
   put '<BODY>';
   put '<BLOCKQUOTE>';
   put '<P><img src="anigif.gif" 
        alt="Microarray animation" ';
   put '</BLOCKQUOTE>';
   put '</BODY>';
   put '</HTML>';

run;
