 /*+-------------------------------------------------------+
   |          SUGI 29 Sample                               |
   |          SAS MAPPING: Tips and tricks                 |
   |                                                       |
   |    NAME: Drillhover.sas                               |
   |   TITLE: Start with zipcode.sas and add tooltips and  |
   |          drill down for both the annotate dots and    |
   |          the states - different info, different drill |
   | PRODUCT: GRAPH                                        |
   |                                                       |
   |   NOTES: Last Update 19FEB04                          |
   |          This example outputs several ODS device types|
   +-------------------------------------------------------+*/


%let name=drillhover;
filename odsout '.';
goptions reset=global ;
/** Sets goptions xpixels and ypixels **/
%inc '../setsize.sas';


/***********************************************************************/    
/* Get your SAS training center location data with zipcode and phone.  */
/* Substitute your own zipcode based location data here.               */
/***********************************************************************/    
data trainzip;
  length zip 5. locationcode $ 2. phone $ 12.;
  /* zipcode of sas training centers, location code and phone */
  input zip  locationcode $ phone $;
  cards;
30326 AT 404-814-2560
78726 AU 512-258-5171
07921 BE 908-470-0080
02199 BO 617-262-4201
97006 BV 503-617-7100
27513 CA 919-677-8000
60601 CH 312-819-6800
45242 CI 513-563-9444
44131 CL 216-642-0641
75207 DA 214-977-3921
48075 DE 248-354-3008
80111 EN 303-290-9112
06033 HA 860-633-4119
77056 HO 713-355-2707
46250 IN 317-578-1700
66210 KC 913-491-1166
92614 LA 949-852-8550
33131 MI 305-381-9980
55402 MN 612-349-9023
10019 NY 212-757-3826
32746 OR 407-804-1995
15222 PB 412-227-6446
85012 PH 602-265-1616
19087 PI 610-640-0940
20850 RO 301-838-7030
98101 SE 206-681-6084
94104 SF 415-421-2227
95131 SJ 408-573-0902
63102 SL 314-421-6364
;
run;



/***********************************************************************/    
/* Sort the data by zipcode and then
/* get the X,Y centroid location for the zipcode by merging with the   */
/* zipcode dataset.  This will give you more information than you want */
/***********************************************************************/    
proc sort data=trainzip; 
  by zip;
run;
data temp(keep= zip locationcode phone x y city);  
  merge trainzip sashelp.zipcode;  
  by zip;
run;


/***********************************************************************/    
/* Now, drop the records that were not in the original data or         */
/* records that don't have a zipcode match. Adjust the lat/long to     */
/* match the map.                                                      */
/***********************************************************************/    
data trainzipxy; 
  set temp; 
    if (locationcode ^='' and x NE . AND y NE .) then do;
	  /*adjust the Lat/long values to match the map*/
      x=atan(1)/45 * x *-1;
      y=atan(1)/45 * y;
      output;
    end;
	/* put out a message for bad zipcodes */
	else if (locationcode ^='' and x=.) then do; /* couldn't find zipcode*/
	   put "WARNING: Check zipcode " zip " because it wasn't located.";
	   end;
run;


/************************************************************************/    
/* Create the annotate dataset with a red 'dot' at each zipcode location*/
/************************************************************************/    
data trainloc;
  set trainzipxy;
  length function style color $ 8 position $ 1 text $ 20 html $1024;
  retain xsys ysys '2' hsys '3' when 'a' text '';
  anno_flag=1; /*so we can separate the datasets later*/

   /**********************************************************/
   /**********************************************************/
   /* Annotate hover/tool tips and drill down created here.  */
   /* Note that this is created in the annotate dataset.     */
   /**********************************************************/

   /* Create a tool-tip and Drill down for the dot */
  html=
   'alt='||
     quote(
     'City: '||trim(left(city))||'0D'x||
	 'Phone: '||trim(left(phone))||'0D'x||
	 'Code: '||trim(left(locationcode))||'0D'x||
     'Zipcode: '||trim(left(zip))||' '
     ) || ' '||
    'href="http://support.sas.com/training/fyi/'||
	      trim(left(lowcase(locationcode)))||'.html" target=_NEW_'
	 ;

   color='red'; size=.6; rotate=360; style='solid'; function='pie';
   position='5'; 
   output;
run;


/***********************************************************************/    
/* Reduce the size of the actual map data.                             */
/***********************************************************************/    
proc greduce data=maps.states out=work.states;
  id state;
run;
/************************************************************************/    
/* Keep only the 48 contigious states and actually write the reduced map*/   
/************************************************************************/    
proc sql;
  create table work.states as select * from work.states
    where fipstate(state) not in ('AK' 'HI' 'PR')
    and density < 4  /* this applies the greduce */
    ;
quit; run;
/***********************************************************************/ 
/***********************************************************************/ 
/* Create tooltips and drill down for each state - separate from dots. */
/* Notice that this is added to your response dataset.  We create a    */
/* dummy response dataset in this case because we were not using one.  */ 
/***********************************************************************/ 
 data responsedata; /* A fake response dataset */
 length htmlvar $1024;
 do state = 1 to 56;
   statename=fipstate(state);
   /*take only valid states*/
   if statename ^= '' then do;
     htmlvar= 'alt='||quote( trim(left('State: '|| trim(left(fipnamel(state))) ))) ||
              'href='||quote('http://www.wunderground.com/US/'||trim(left(statename))) || 
			        'target=_new_'
              ;
	 output;
   end;
 end;

run;


/***********************************************************************/    
/* Combine the map and annotate Data to project it the same.           */
/* Then split it apart again for display.                              */
/***********************************************************************/    
data work.combined; /* combine data*/
  set work.states trainloc; 
run;
/*project both datasets*/
proc gproject data=work.combined out=work.combined dupok;
  id state;
run;
/*split the data*/
data work.states trainloc;
  set work.combined;
  if anno_flag=1 then output trainloc;
  else output work.states;
run;

/* ---------------------------------------------------------*/     
/* Declare a macro to do the plot, for the specified device */
/* ---------------------------------------------------------*/     
%macro do_map(dev);

  GOPTIONS DEVICE=&dev;
  ODS LISTING CLOSE;
  ODS HTML path=odsout body="&name._&dev..html"
    parameters=('DataTipHighlightColor'='#00ff00')   /* color of outline when you mouse over an area in javameta */
    style=statistical
    ;

  pattern v=s c=CXE9E8DC r=100;
  goptions border;

  goptions htitle=2.75 htext=1.3;
  Title 'SAS Training Sites with tooltips and drilldown';
  
  /******************************************************************/
  /******************************************************************/
  /* Note that response data and annotate data with tool tips and   */
  /* drilldown are used.  Also, html=htmlvar is used in the choro   */
  /* statement for the response data.                               */
  /******************************************************************/
  proc gmap map=work.states data=work.responsedata anno=work.trainloc;
     id state; 
     choro state / coutline=black name="&name" html=htmlvar nolegend;
  run;
  quit;


  ODS HTML CLOSE;
  ODS LISTING;
 
%mend do_map;
/* ---------------------------------------------------------*/     

/***********************************************************************/
/* Run GMAP do 'display' the map.                                      */
/* Call the macro, for each desired 'device' type                      */
/***********************************************************************/
%do_map(gif);
/*
%do_map(java);
%do_map(activex);
*/
