 /*+-------------------------------------------------------+
   |          SUGI 29 Sample                               |
   |          SAS MAPPING: Tips and tricks                 |
   |                                                       |
   |    NAME: dotdensity.sas                               |
   |   TITLE: Simulate a dot-density map.                  |
   |          This example takes zipcode location data and |
   |          finds the zipcode centroid location.  Then   |
   |          it randomizes the locations so that the      |
   |          same location doesn't fall at the same spot. |
   | PRODUCT: GRAPH                                        |
   |                                                       |
   |   NOTES: Last Update 24FEB04                          |
   |          This example outputs several ODS device types|
   |          Dot size may need to change for different    |
   |          ODS devices.                                 |
   +-------------------------------------------------------+*/


%let name=dotdens;
filename odsout '.';
goptions reset=global ;
/** Sets goptions xpixels and ypixels **/
%inc '../setsize.sas';


/***********************************************************************/    
/* Illness location by zipcode                                         */
/* Substitute your own zipcode based location data here.               */
/***********************************************************************/    
data address;
   infile 'zip.out';
   length zip 5.;
      input zip ;  /*must have variable 'zip' for matching*/
      addr_data=1; /*unique var for separating data later*/
run;

%macro ziplocation(indata,outdata,uniquevar,radians);
/*---------------------------------------------------------------------*/ 
/* Find the x,y location of the zipcode centroid.                      */
/*                                                                     */ 
/* Sort the data by zipcode and then                                   */
/* get the X,Y centroid location for the zipcode by merging with the   */
/* zipcode dataset.  This will give you more information than you want */
/* so you drop the records not in the original address data.           */
/* Arguments:                                                          */
/*   indata= input dataset with zipcode data to lookup x,y             */
/*   outdata= output dataset created with location information         */
/*   uniquevar= variable name used only in input data so you can       */
/*              eliminate other data.                                  */
/*   radians= set to 1 if the map data has x,y in radians.  zipcode    */
/*            data is in lat/long degrees.                             */
/*---------------------------------------------------------------------*/   
proc sort data=&indata; 
  by zip;
run;
data temp(keep= zip &uniquevar x y city);  
  merge address sashelp.zipcode;  
  by zip;
run;


/***********************************************************************/    
/* Now, drop the records that were not in the original data or         */
/* records that don't have a zipcode match. Adjust the lat/long to     */
/* match the map.                                                      */
/***********************************************************************/    
data &outdata; 
  set temp; 
    if (&uniquevar ^= . and x NE . AND y NE .) then do;
	  /*adjust lat/long degrees to radians, if needed*/
	  %if &radians %then %do;
        x=atan(1)/45 * x *-1;
        y=atan(1)/45 * y;
	  %end;
      output;
    end;
	/* put out a message for bad zipcodes */
	else if (&uniquevar ^=. and x=.) then do; /* couldn't find zipcode*/
	   put "WARNING: Check zipcode " zip " because it wasn't located.";
	   end;
run;
%mend;

%macro dot_density(indata,outdata,spread,dotsize,color,idvar);
/*----------------------------------------------------------------------*/    
/* Create an annotate dataset with a randomized 'dot' a each location.  */
/* This simulates a dot-density map so that all dots at the same        */
/* location do not appear directly on top of each other.  The user      */
/* must adjust spread and dotsize for their map needs.                  */
/*                                                                      */
/* Arguments:                                                           */
/* indata= input dataset; outdata= output dataset;                      */
/* spread= offset from center point in any direction.                   */
/* dotsize= size of dot for each value                                  */
/* color= color of dot;  idvar= variable to use in tooltip              */
/* Notes:  Dotsize and spread must change depending on the scale of the */
/*         Map, whether you are using degrees or radians and            */
/*         the number of items you are showing. The dot size may also   */
/*         vary between different types of output (eg, Gif, Java, actx) */
/* Warning:  Dot-density maps (or dot maps) are not 'accurate' because  */
/* of the random nature of the map and only show patterns.  Accuracy is */
/* better when you are showing a smaller area and fewer items.          */ 
/*----------------------------------------------------------------------*/    
data &outdata;
  set &indata;
  length function style color $ 8 position $ 1 text $ 20 html $1024;
  retain xsys ysys '2' hsys '3' when 'a' text '' ;
  retain rotate 360 style 'solid' function 'pie' position '5';
  anno_flag=1; /*so we can separate the datasets later*/
  color=&color; 
  size=&dotsize; 
  spread=&spread;

  /* create randomness around centroid so you can see more than one dot*/
  x=x- spread + (spread*2)*rannor(0);
  y=y- spread + (spread*2)*rannor(0);

   /* Create a tool-tip for the dot */
  html=
   'alt='||
     quote(
     "&idvar : "||trim(left(&idvar))||' '
     );
   output;
run;
%mend;

/***********************************************************************/    
/* Match zipcodes to an x,y location                                   */
/***********************************************************************/  
%ziplocation(address,zipxy,addr_data,1);

/***********************************************************************/    
/* Create a 'dot-density' annotate dataset                             */
/***********************************************************************/  
%dot_density(zipxy,dotanno,.0001,.1,'red',zip);

/***********************************************************************/    
/* Reduce the size of the actual map data.                             */
/***********************************************************************/  
proc greduce data=maps.counties out=work.states;
  id state;
run;

/************************************************************************/    
/* Keep only 3 counties in NC                                           */   
/************************************************************************/    
proc sql;
  create table work.states as select * from work.states
    where fipstate(state) EQ ('NC')
	and (county=183 or county=85 or county=101)
    and density < 4  
    ;
quit; run;



/***********************************************************************/    
/* Combine the map and annotate Data to project it the same.           */
/* Then split it apart again for display.                              */
/***********************************************************************/    
data work.combined; /* combine data*/
  set work.states dotanno; 
run;
/*project both datasets*/

proc gproject data=work.combined out=work.combined dupok;
  id state;
run;

/*split the data*/
data work.states dotanno;
  set work.combined;
  if anno_flag=1 then output dotanno;
  else output work.states;
run;

/* ---------------------------------------------------------*/     
/* Declare a macro to do the plot, for the specified device */
/* ---------------------------------------------------------*/     
%macro do_map(dev);

  GOPTIONS DEVICE=&dev;
  ODS LISTING CLOSE;
  ODS HTML path=odsout body="&name._&dev..html"
    parameters=('DataTipHighlightColor'='#00ff00')   /* color of outline when you mouse over an area in javameta */
    style=statistical
    ;

  pattern v=s c=CXE9E8DC r=100;
  goptions border;

  goptions htitle=2.75 htext=1.3;
  Title 'Dot-density Map of Flu Outbreak by Zipcode';
  
  proc gmap data=work.states map=work.states anno=work.dotanno;
     id county; 
     choro county / coutline=black name="&name" nolegend;
  run;
  quit;


  ODS HTML CLOSE;
  ODS LISTING;
 
%mend do_map;
/* ---------------------------------------------------------*/     

/***********************************************************************/
/* Run GMAP do 'display' the map.                                      */
/* Call the macro, for each desired 'device' type                      */
/***********************************************************************/
%do_map(gif);

/*
%do_map(javameta);
%do_map(java);
%do_map(activex);
%do_map(javaimg);
%do_map(actximg);
*/

