 /*+-------------------------------------------------------+
   |          SUGI 29 Sample                               |
   |          SAS MAPPING: Tips and tricks                 |
   |                                                       |
   |    NAME: gismap.sas                                   |
   |   TITLE: Simulate a GIS looking map with streets      |
   |          This example reads in shapefile data         |
   |          containing zipcode boundries and streets     |
   |          and draws the streets for the Cary NC area.  |
   |          This program highlights how to change the    |
   |          street data into an annotate dataset.        |
   |          This shapefile data is freely available for  |
   |          download from the Wake County government.    |
   | PRODUCT: GRAPH                                        |
   |                                                       |
   |   NOTES: Last Update 24FEB04                          |
   |          This example outputs several ODS device types|
   |          If 'fuzz' is set to anything but zero, you   |
   |          you will get warnings about annotate         |
   |          observations being off the map.              |
   +-------------------------------------------------------+*/


%let name=gis;
filename odsout '.';
goptions reset=global ;
%let fuzz=0.001;  /*allow streets to be slightly off the map area*/

/** Sets goptions xpixels and ypixels **/
%inc '../setsize.sas';

/***********************************************************************/    
/* Import the zipcode map and street data.                             */
/* The shapefile data comes from Wake County GIS MAPS web site:        */
/*    http://lnweb02.co.wake.nc.us/gis/gismaps.nsf                     */
/***********************************************************************/ 
proc mapimport out=sasuser.map datafile='zipcodes.shp';
run;
proc mapimport out=sasuser.street datafile='streets.shp';
     select stname stype length class;
run; 

/***********************************************************************/
/* Subset only the CARY, NC zipcodes out for the map.                  */  
/***********************************************************************/
data sasuser.map;
   retain xmax 0 ymax 0 xmin 9999999 ymin 9999999;
   set sasuser.map (where= (ZIPNAME='CARY'));
   /*find the max, min values for the map so you can clip roads*/
   xmax=max(xmax,x);
   ymax=max(ymax,y);
   xmin=min(xmin,x);
   ymin=min(ymin,y);
   run;

/* assign max,min values to macro variables for later use */
data temp;
   set sasuser.map end=last;
   if last then do;
        call symput('xmax',xmax);
	  call symput('ymax',ymax);
	  call symput('xmin',xmin);
	  call symput('ymin',ymin);
	  end;
	  run;

/***********************************************************************/
/*** Create an annotate dataset from the road data.                  ***/
/***********************************************************************/
data sasuser.roadanno;
  set sasuser.street;
  length function style color $ 8 text $ 25 prevroad $ 40;
  retain hsys '3' xsys ysys '2' line 1 prevroad '0' prevroadlen 0;
  retain style '' text '' when 'A' position '6' size .01 color 'red';

  if x > (&xmax + (&xmax * &fuzz)) or x < (&xmin - (&xmin * &fuzz)) or
     y > (&ymax + (&ymax * &fuzz)) or y < (&ymin - (&ymin * &fuzz)) 
     then do; /*skip values off the map*/
          end;
  else
  /* Is this the start of a road or have we been drawing it already*/
  if ((prevroad ne STNAME) or
       (prevroad eq STNAME and  LENGTH ne prevroadlen )) then do;
	   /*if the street 'begins with' - Interstate or US hwy, change color */
	   if stname =: 'I 4' or stname =: 'I 5' then do; /*Interstate*/
              color='red';
		  size='.07';
		  end;
	   else if stname =: 'US' or class < 6 then do; /*major highways*/
              color='blue';
		  size='.04';
		  end;
	   else do;  /* the rest of the smaller roads */
              color='black';
		  size='.01';
		  end;
         
         function='move';  /* start a new road */
         output;
         /* save for next time */
         prevroad=STNAME;  
         prevroadlen = LENGTH;
         end;

  else do; 
       function='draw';
       output;
       end;
run;




/* ---------------------------------------------------------*/ 
/* Declare a macro to do the map, for the specified device  */
/* ---------------------------------------------------------*/     
%macro do_map(dev);

  GOPTIONS DEVICE=&dev;
  ODS LISTING CLOSE;

  ods html
     path=odsout 
     body="&name._&dev..html"
     style=curve;

  pattern v=s c=CXE9E8DC r=100;  /*make all map regions the same color*/
  Title1 font='Helvetica' /*h=14pt*/ "Annotated Streets for Cary, NC";

  proc gmap map=sasuser.map  data=sasuser.map all anno=sasuser.roadanno ;
    id zip_id; 
    choro zip_id / des='Cary zipcodes' nolegend; 
  run;

ods html close;
ods listing;
%mend do_map;
/* ---------------------------------------------------------*/     

/***********************************************************************/
/* Run the Macro - thus running Proc GMAP.                             */
/***********************************************************************/
/* Call the macro, for each desired 'device' type */
%do_map(gif);
%do_map(javameta);

/*Because of memory issues, don't run with Java/activex*/




