 /*+-------------------------------------------------------+
   |          SUGI 29 Sample                               |
   |          SAS MAPPING: Tips and tricks                 |
   |                                                       |
   |    NAME: microarray.sas                               |
   |   TITLE: Create a microarray for DNA results.         |
   |          Create a map that is a grid of circles.  The |
   |          circles are colored Red, Yellow, Green and   |
   |          black depending on the value of the results. |
   | PRODUCT: GRAPH                                        |
   |                                                       |
   |   NOTES: Last Update 24FEB04                          |
   |          This example outputs several ODS device types|
   +-------------------------------------------------------+*/


%let name=microarray;
filename odsout '.';

/*cellsize of 1 has no padding between circles*/
%let cellsize=.75;  /*a number greater than zero and less than/equal 1*/
%let MAXCOLS=14;    /*grid size*/
%let MAXROWS=14;
%let CIRCLEINC=36;  /*10 points on the circle*/


goptions reset;
/** Sets goptions xpixels and ypixels **/
%inc '../setsize.sas';


/***********************************************************************/    
/* Create a map with round cells by the specified grid size.           */
/***********************************************************************/  
data mapdata (keep=x y location);
retain location 0;
/* 0,0 is the bottom left, so output in this order to */
/* make 1,1 the top left and 2,1 to be the second row, first col, etc*/
do b = &MAXROWS to 1 by -1;
   do a =1 to &MAXCOLS;
       location=location+1;  /*assign location in sequential order*/
       radius = &cellsize/2; 
       xcenter=a+radius;
       ycenter=b+radius;
       do degrees=0 to 360 by &CIRCLEINC;  
         radians=degrees/57.3; 
         x=(radius * cos(radians)) + xcenter;
         y=(radius * sin(radians)) + ycenter;
         output;
       end;
   end;
end;


/***********************************************************************/    
/* Read in response data to display on the grid and process it         */
/***********************************************************************/  
data sasuser.arraydata; /* What was tested */
    infile 'array.txt';
    retain location 0;
    length description $100;  length genbank $10.;
    input metarow 1-2 metacol 3-4 arrayrow 5-7 arraycol 8-10 symbol $ 12-19 description $ 20-116 genbank $  ;
run;

/*drop all but the first metaarray and add fake test results*/
data response (drop=metarow metacol); 
   length htmlvar $1024;
   set sasuser.arraydata;
   if (metarow = 1 AND metacol = 1) then do;
       location= ((arrayrow-1)* &MAXCOLS ) + arraycol;
       /* Generate fake random data */
       outcome1 =(ranuni(2) * 4 );  
       outcome2 =(ranuni(2) * 4 ); 
       outcome3 =(ranuni(2) * 4 ); 
       outcome4 =(ranuni(2) * 4 ); 
       mean = (outcome1+outcome2+outcome3+outcome4)/4;
       wtmean= (outcome1* .5)+(outcome2* .2) + (outcome3* .2) + (outcome4 * .1);
       duration =int(ranuni(2) * 20 +1);
       value= int((ranuni(2) * 4 )+1);

       /* tooltips and drill down */
       htmlvar= 'alt='||quote( trim(left('Symbol: '|| trim(left(Symbol)) )) ||'0D'x||
                         trim(left('Genbank: '|| trim(left(genbank)) )) ||'0D'x||
                         trim(left('Description '|| trim(left(description)) )) ) ||
          'href='||quote('http://www.sas.com');

       output;
      end;
   run;



/***********************************************************************/    
/* Setup 4 colors and legend                                           */
/***********************************************************************/  
/*set our red, yellow, green, black colors*/
pattern1  v=s c=cxff0000;
pattern2  v=s c=cxffff00;
pattern3  v=s c=cx00ff00;
pattern4  v=s c=cx000000;

legend1 
   position=(middle left) 
   across=1 
   offset=(3pct,0pct)
   frame cframe=gray 
   value=(color=white h=2.5pct

    tick=1 'sample DNA'
    tick=2 'combination'
    tick=3 'control DNA'
    tick=4 'neither'
   )
   /* shape of legend chicklets ... width, height */
   /* if you change the font, you'll have to adjust shape */
   shape=bar(3,1) 
   label=(color=white position=(top center) 'Color Key')
  ;


/* ---------------------------------------------------------*/     
/* Declare a macro to do the map, for the specified device  */
/* ---------------------------------------------------------*/     
%macro do_map(dev);

   GOPTIONS DEVICE=&dev cback=black ctext=white;
   ODS LISTING CLOSE;
   ODS HTML path=odsout body="&name._&dev..html"
   style=d3d
   ;
   goptions ftitle=zapfb htitle=5pct ftext='arial' htext=3pct;

   Title1 "MicroArray Demo";
   proc gmap map=mapdata data=response;
    id location;
    choro value /
       html=htmlvar  /* this variable controls the flyover & drilldown */
       des=""  /* to suppress the v9 default alt= flyover for entire graphics area */
       name="&name"  /* name for the gif file */
       legend=legend1 
       ; 

   run;

   quit;
   ODS HTML CLOSE;
   ODS LISTING;
 
%mend do_map;
/* ---------------------------------------------------------*/     


/**********************************************************************/
/* Create the map with Proc GMAP.                                     */
/**********************************************************************/

/* Call the macro, for each desired 'device' type */
%do_map(gif);

%do_map(java);
%do_map(activex);
%do_map(javameta);
%do_map(actximg);
%do_map(javaimg);

   
   
      
      