 /*+-------------------------------------------------------+
   |          SUGI 29 Sample                               |
   |          SAS MAPPING: Tips and tricks                 |
   |                                                       |
   |    NAME: perception.sas                               |
   |   TITLE: Create a perception grid.                    |
   |          Create map data for the grid and then map    |
   |          the response data to a location in the grid  |
   |          and annotate the location.                   |
   | PRODUCT: GRAPH                                        |
   |                                                       |
   |   NOTES: Last Update 24FEB04                          |
   +-------------------------------------------------------+*/


/***********************************************************************/
/* Values to set up the size of the grid and the range of response data*/
/***********************************************************************/

%let maxx=.7;       /*max grid size in x direction for one block*/
%let maxy=.5;       /*max grid size in y direction for one block*/
%let maxvalue=10;   /*value range from 1 to 10*/
%let maxservice=10; /*service range from 1 to 10*/

%let name=perception;
filename odsout '.';
goptions reset=global ;
/** Sets goptions xpixels and ypixels **/
%inc '../setsize.sas';


/***********************************************************************/
/* Response data to plot on map.  The data represents customer         */
/* responses about the 'value' and 'service' of the restaurant on a    */
/* scale of 1 to 10 (10 is the best).                                  */
/***********************************************************************/
data restaurant;
   length name $30 value 8. service 8.;
   input value service name ;
   cards;
6  6  Alfredos
9  7  Belles
3  7  Danny's
1  1  Joes
;


/***********************************************************************/
/* Create an annotate dataset for the Response data. Draw circles that */
/* represent the rating of the restaurant.  Label the restaurant name  */
/* and put up tool tips when you hover over the circles.               */
/***********************************************************************/
   data anno_rest;
     length function style color $ 8 position $ 1 text $ 20 html $1024;
     retain xsys ysys '2' hsys '3' when 'a';
     set work.restaurant;
     anno_flag=1;
     /* convert values to x,y locations on the map */
     x=(service / &maxservice) * (2* &maxx);
     y=(value / &maxvalue) * (2 * &maxy);
     text='';
     /* Create a chart-tip for the dot & label */
     html=
      'alt='||
        quote(
         'Name: '||trim(left(name))||'0D'x||
         'Value: '||trim(left(value))||'0D'x||
         'Service: '||trim(left(service))||' '
         );
     /* output a dot for each restaurant based on the 2 values */
     color='cxffffff'; /*white*/
     function='pie';
     size=(25 / &maxvalue) ; /*vary the size depending on range of values */
     rotate=360; style='solid'; 
     position='5'; 
     output;
 
	 /* output the name of the restaurant */
     color='cxff0000'; /*red*/
     function='label'; 
     size=(35 / &maxvalue) ; /*vary size with dot size*/
     y=y-(size/1000);  /*adjust location based on dot size*/
     text=name; position='8'; style="'arial'";
     output;
 
run;

/***********************************************************************/
/* Create an annotate dataset for labeling the map.                    */
/***********************************************************************/
data anno_label;
   length function style color $ 8 position $ 1 text $ 20 html $1024;
   retain xsys ysys '2' hsys '3' when 'a';
   anno_flag=1;
   function='LABEL'; style="'Arial'"; 
   size=(35 / &maxvalue);
   text='Good';    y=(&maxy+&maxy); x=0; position='4'; output;
   text='Poor';    y=0; x=0; position='1';output;
   text='Slow';    y=0; x=0; position='9'; output;
   text='Fast';    y=0; x=(&maxx+&maxx); position='7'; output;
   text='Service'; y=0; x=&maxx; position='8'; size=size * 1.25; output;
   text='Value';   y=&maxy; x=0; position='2'; angle=90; output;
   
   run;

/***********************************************************************/
/* Combine the annotate datasets.                                      */
/***********************************************************************/
data anno;
  set anno_rest anno_label;
  run;


/***********************************************************************/
/* Macro to simplify the creation of the blocks of the map.            */
/***********************************************************************/
%macro square(x1,y1,sizex,sizey);
  x=x1; y=y1; output;
  x=x1+ &sizex; y=y1; output;
  x=x1+ &sizex; y=y1+ &sizey; output;
  x=x1; y=y1+ &sizey; output;
  x=x1; y=y1; output;
%mend;

/***********************************************************************/
/* Create the map data based on the sizes given.                       */
/***********************************************************************/
data map;
  id=0;
  do x1=0 to &maxx by &maxx;
     do y1=0 to &maxy by &maxy;
	    id+1;
	    %square(x1,y1,&maxx,&maxy);
		end;
	end;
run;


/***********************************************************************/
/* Put a fake map area to the bottom/left of the map, to give room     */
/* for the annotated labels on the left & bottom.                      */
/* Combine this 'map' with your previous map.                          */
/***********************************************************************/
data work.fake;
  id=0; 
  x=-.2; y=-.2; output;
  x=x- .000000001; y=y+.000000001; output;
  x=x+ .000000001; output;
run;
data work.map; 
  set work.map work.fake;
run;


/* ---------------------------------------------------------*/     
/* Declare a macro to do the map, for the specified device */
/* ---------------------------------------------------------*/     
%macro do_map(dev);
   GOPTIONS DEVICE=&dev;
   ODS LISTING CLOSE;
   ODS HTML path=odsout body="&name._&dev..html"
     parameters=('DataTipHighlightColor'='#00ff00')   /* color of outline when you mouse over an area in javameta */
     ;

   pattern v=s c=CX90f090 r=100;
   goptions cback=ltgray;
   Title "Perception Map for Restaurants";
   proc gmap data=map map=map anno=anno; 
     id id; choro id / nolegend  coutline='black'; run;quit;
   ODS HTML CLOSE;
   ODS LISTING;
 
%mend do_map;
/* ---------------------------------------------------------*/     


/***********************************************************************/
/* Draw the map.                                                       */
/***********************************************************************/
%do_map(gif);

/*
%do_map(java);
%do_map(activex);
*/
