 /*+-------------------------------------------------------+
   |          SUGI 29 Sample                               |
   |          SAS MAPPING: Tips and tricks                 |
   |    NAME: Polarplot.sas                                |
   |   TITLE: Create an ODS based Polar Plot with annotate |
   | PRODUCT: GRAPH                                        |
   |                                                       |
   |   NOTES: Last Update 05Jun03                          |
   |          Note:  this displays on top of a 'map' which |
   |                 is essentially empty.                 |
   |          Original Author:  Richard A. DeVenezia 2/2000|
   |                   used datastep graph functions.  This|
   |                   was modified from that.             |
   +-------------------------------------------------------+*/

/*****************************************************/
/*** Setup names and default values                ***/
/*****************************************************/
%let name=polar;
filename odsout '.';
%let chartsize=.2;
%let textsize=2;
%let plotsize=.5;
%let chartcolor='RED';
%let textcolor='BLACK';
%let plotcolor='BLACK';

%let pi = 3.1415926;

goptions reset=global;


/*****************************************************/
/*** Generate response data.                       ***/
/*****************************************************/

/*** Assume a day is a 'circle' ***/
data oneday;
  * generate fake every four minutes data;
  today = intnx ('DTDAY', datetime (), 0);
  what = ranuni (1);
  do hour = 0 to 23;
    do qh = 0 to 59 by 4;
      whendate = today + hour * 3600 + qh * 60;
      what = what + ranuni (0) - ranuni (0);
      output;
    end;
  end;
  attrib
    whendate label = 'When was measurement made' format=datetime19.
    what label = 'The measurement'           format=7.3
  ;
  keep whendate what ;
run;

/*** Determine least 'what', do not what to plot a negative radius ***/
/*** Least 'what' will become the '0' radius.                      ***/
/*** Store results in Macro Variables.                             ***/
proc sql noprint;
  select
    intnx('dthour', min(whendate),0) format=best12.
  , intnx('dthour', max(whendate),1) format=best12.
  , min(what)
  , max(what)
  into :minWhen, :maxWhen, :minWhat, :maxWhat
  from oneday;
quit;

/*** Convert one day to clock like polar coordinates        ***/
data onedayP;
  set oneday;
  r = what;
  theta = (whendate - &minWhen) / (&maxWhen - &minWhen) * 2 * &pi;
  theta = &pi / 2 - theta;
  attrib
    r     label = 'The measurement'           format=7.3
    theta label = 'When was measurement made'
  ;
run;



goptions goutmode=append;
goptions hsize=5in vsize=5in;


/*** Calculate XY values for theta reference lines ***/
%macro XY(n,r,theta);
  %* assume rMin exists and value is where origin should be;
  x&n = (&r-rMin)*cos(&theta);
  y&n = (&r-rMin)*sin(&theta);
%mend;

/*****************************************************/
/*** Macros to do the annotation work.             ***/
/*****************************************************/

/*** Annotate - Move to starting point ***/
%macro moveto(x,y);
    function='MOVE';
    x=&x; y=&y;
    output;
%mend;

/*** Annotate - Draw to next point ***/
%macro drawto(x,y);
    function='DRAW';
    x=&x; y=&y;
    output;
%mend;

/*** Annotate - Write a label ***/
%macro labelat(x,y,label);
    function='LABEL';
    x=&x; y=&y;
    color=&textcolor;
    size=&textsize;
    text=&label;
    position='5'; /*center*/
    output;
    color=&chartcolor;
    size=&chartsize;
    position='6';
%mend;

/*** Move and Draw to draw a line***/
%macro gdrawline(x1,x2, y1,y2, label);
    %moveto(&x1,&y1);
    %drawto(&x2,&y2);
    %labelat(&x2,&y2,&label);
%mend;

/*** Label the arc ***/
%macro labelarc(centerx,centery, radius, degrees, incr, label);
     radians=&degrees/57.3;
     x=(&radius * cos(radians)) + &centerx;
     y=(&radius * sin(radians)) + &centery;
     %labelat(x,y,&label);
%mend;

/*** Draw an arc the hard way ***/
%macro gdrawarc(centerx,centery, radius, start, end, incr);
    do degrees=&start to &end by &incr;
         radians=degrees/57.3;
         x=(&radius * cos(radians)) + &centerx;
         y=(&radius * sin(radians)) + &centery;
         if (degrees = &start) then do;
            function='MOVE';
            output;
            end;
         function='DRAW';
         output;
         if (degrees = 90) then do;
            %labelat(x,y,&radius);
            end;
       end;
%mend;



/*** create a tiny triangle ***/
%macro tinymap(x,y,id);
   id=&id;
   x=&x; y=&y; output;
   x=x+0.00001;y=&y;output;
   x=&x; y=y+0.00001; output;
   x=&x; y=&y; output;
%mend;

/*****************************************************/
/*** Create an 'empty' map dataset so you can      ***/
/*** Anotate over that area.  Create tiny polygons ***/
/*** at each extreme side to size the map space    ***/
/*** The map itself is never used.                 ***/
/*****************************************************/                           
data polarmap;
  rMin = &minWhat - .10 * (&maxWhat-&minWhat);
  rMax = &maxWhat + .10 * (&maxWhat-&minWhat);
  rRange = rMax - rMin ;
  %tinymap(rRange,rRange,1);
  %tinymap(rRange,-rRange,2);
  %tinymap(-rRange,rRange,3);
  %tinymap(-rRange,-rRange,4);
run;

/*****************************************************/
/*** Create an annotate dataset of the polar chart.***/
/*** Draw the data on top of the chart.            ***/
/*****************************************************/
data polaranno;
length function style color $ 8 text $ 2;
  retain hsys '3' xsys ysys '2' line 1 ;
  retain style 'empty' text '' when 'A' position '6' size .2;
  color= &chartcolor;
  size=&chartsize;
  
  * fudge radial offset away from center;
  rMin = &minWhat - .10 * (&maxWhat-&minWhat);
  rMax = &maxWhat + .10 * (&maxWhat-&minWhat);
  rRange = rMax - rMin ;

  line=33;  /* dotted line*/

  * plot some theta reference lines;
  nRref = 6;  /*radial reference lines*/
  nTref = 24; /*hours*/
  do i = 1 to nTref;
    theta = (i-1) / nTref * 2 * &pi ;
    %XY(1,&minWhat,theta);
    %XY(2,rMax,theta);
    /*adjust the label for the time*/
    lbl=(nTref/4)-(i-1);
    if (lbl < 0) then lbl=nTref+lbl;
    %gdrawline(x1, x2, y1, y2, lbl);
    end;
  
  * plot some radial reference lines;
  do i = 1 to nRref;
     r = i / nRref * (&maxWhat-&minWhat);
     %gdrawarc (0,0, r, 0, 360,nTref/2);  
     %labelarc(0,0,r,90,nTref/2,r+&minWhat);

     end;


  * plot the data radial reference lines;
  color=&plotcolor;
  size=&plotsize;
  line=1;
  n = 1;
  eod = 0;
  do while (not(eod));
    set onedayP end=eod;
    %XY(2,r,theta);
    if (n = 1) then do; 
       %moveto(x2,y2);
       end;
    else
      %drawto(x2,y2);
    n+1;
  end;


/*****************************************************/
/*** Macro outputs the chart for each ODS device.  ***/
/*****************************************************/
%macro do_map(dev);

  GOPTIONS DEVICE=&dev;
  ODS LISTING CLOSE;
  ods html
     path=odsout 
     body="&name._&dev..html";

   pattern v=s c=CXffffff r=100;
   title "Measurements for a day";

   /*** display the empty map and annotate over it ***/
   proc gmap data=polarmap map=polarmap;
    id id; choro id / annotate=polaranno nolegend;
  run;
  ods html close;
  ods listing;
%mend do_map;


/*****************************************************/
/*** Plot for each ODS device type                 ***/
/*****************************************************/

%do_map(gif);
*%do_map(java);
*%do_map(activex);
*%do_map(actximg);
*%do_map(javaimg);


